import codepipeline = require('@aws-cdk/aws-codepipeline-api');
import cdk = require('@aws-cdk/cdk');
import { IProject } from './project';
/**
 * Common construction properties of all CodeBuild Pipeline Actions.
 */
export interface CommonCodeBuildActionProps {
    /**
     * The list of additional input Artifacts for this Action.
     */
    readonly additionalInputArtifacts?: codepipeline.Artifact[];
    /**
     * The list of names for additional output Artifacts for this Action.
     * The resulting output artifacts can be accessed with the `additionalOutputArtifacts`
     * method of the Action.
     */
    readonly additionalOutputArtifactNames?: string[];
}
/**
 * Common properties for creating {@link PipelineBuildAction} -
 * either directly, through its constructor,
 * or through {@link IProject#toCodePipelineBuildAction}.
 */
export interface CommonPipelineBuildActionProps extends CommonCodeBuildActionProps, codepipeline.CommonActionProps {
    /**
     * The source to use as input for this build.
     */
    readonly inputArtifact: codepipeline.Artifact;
    /**
     * The name of the build's output artifact.
     *
     * @default an auto-generated name will be used
     */
    readonly outputArtifactName?: string;
}
/**
 * Construction properties of the {@link PipelineBuildAction CodeBuild build CodePipeline Action}.
 */
export interface PipelineBuildActionProps extends CommonPipelineBuildActionProps {
    /**
     * The build project
     */
    readonly project: IProject;
}
/**
 * CodePipeline build Action that uses AWS CodeBuild.
 */
export declare class PipelineBuildAction extends codepipeline.BuildAction {
    private readonly props;
    constructor(props: PipelineBuildActionProps);
    /**
     * Returns the additional output artifacts defined for this Action.
     * Their names will be taken from the {@link CommonCodeBuildActionProps#additionalOutputArtifactNames}
     * property.
     *
     * @returns all additional output artifacts defined for this Action
     * @see #additionalOutputArtifact
     */
    additionalOutputArtifacts(): codepipeline.Artifact[];
    /**
     * Returns the additional output artifact with the given name,
     * or throws an exception if an artifact with that name was not found
     * in the additonal output artifacts.
     * The names are defined by the {@link CommonCodeBuildActionProps#additionalOutputArtifactNames}
     * property.
     *
     * @param name the name of the artifact to find
     * @returns the artifact with the given name
     * @see #additionalOutputArtifacts
     */
    additionalOutputArtifact(name: string): codepipeline.Artifact;
    protected bind(stage: codepipeline.IStage, _scope: cdk.Construct): void;
}
/**
 * Common properties for creating {@link PipelineTestAction} -
 * either directly, through its constructor,
 * or through {@link IProject#toCodePipelineTestAction}.
 */
export interface CommonPipelineTestActionProps extends CommonCodeBuildActionProps, codepipeline.CommonActionProps {
    /**
     * The source to use as input for this test.
     */
    readonly inputArtifact: codepipeline.Artifact;
    /**
     * The optional name of the primary output artifact.
     * If you provide a value here,
     * then the `outputArtifact` property of your Action will be non-null.
     * If you don't, `outputArtifact` will be `null`.
     *
     * @default the Action will not have an output artifact
     */
    readonly outputArtifactName?: string;
}
/**
 * Construction properties of the {@link PipelineTestAction CodeBuild test CodePipeline Action}.
 */
export interface PipelineTestActionProps extends CommonPipelineTestActionProps {
    /**
     * The build Project.
     */
    readonly project: IProject;
}
export declare class PipelineTestAction extends codepipeline.TestAction {
    private readonly props;
    constructor(props: PipelineTestActionProps);
    /**
     * Returns the additional output artifacts defined for this Action.
     * Their names will be taken from the {@link CommonCodeBuildActionProps#additionalOutputArtifactNames}
     * property.
     *
     * @returns all additional output artifacts defined for this Action
     * @see #additionalOutputArtifact
     */
    additionalOutputArtifacts(): codepipeline.Artifact[];
    /**
     * Returns the additional output artifact with the given name,
     * or throws an exception if an artifact with that name was not found
     * in the additonal output artifacts.
     * The names are defined by the {@link CommonCodeBuildActionProps#additionalOutputArtifactNames}
     * property.
     *
     * @param name the name of the artifact to find
     * @returns the artifact with the given name
     * @see #additionalOutputArtifacts
     */
    additionalOutputArtifact(name: string): codepipeline.Artifact;
    protected bind(stage: codepipeline.IStage, _scope: cdk.Construct): void;
}
